import Promise from 'bluebird';
/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
export function up() {
    const db = this.connection.db;
    return Promise.resolve()
        .then(() => {
            const usersDb = db.collection('users');
            return usersDb.find({ subscriptionStatus: { $exists: true } }).toArray()
                .then(users => Promise.map(users, user => {
                    const status = user.subscriptionStatus;
                    const updatedAt = user.subscriptionUpdatedAt;
                    const customerIdentifier = user.customerIdentifier;
                    const type = 'PAYGO';
                    const subscription = {
                        status,
                        updatedAt,
                        type
                    };
                    if (customerIdentifier) {
                        subscription.customerIdentifier = customerIdentifier;
                    }
                    usersDb.update({ _id: user._id }, { $set: { subscription }, $unset: { subscriptionStatus: '', subscriptionUpdatedAt: '', customerIdentifier: '' } });
                }));
        });
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
export function down() {
    return Promise.reject(new Error('Irreversible change'));
}
