import Promise from 'bluebird';
/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
export function up() {
    const db = this.connection.db;
    return Promise.resolve()
        .then(() => {
            const dataBrokers = db.collection('databrokers');
            return dataBrokers.aggregate([
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user',
                        foreignField: '_id',
                        as: 'userData'
                    }
                },
                {
                    $unwind: '$userData'
                },
                {
                    $group: {
                        _id: '$user',
                        dataBrokers: {
                            $addToSet: { _id: '$_id' }
                        },
                        awsAccountId: { $first: '$userData.awsAccountId' }
                    }
                },
                {
                    $project: {
                        _id: true,
                        dataBrokers: true,
                        awsAccountId: true
                    }
                }
            ]).toArray()
                .then(resultArr => Promise.map(resultArr, result => dataBrokers.update({ $or: result.dataBrokers }, { $set: { awsAccountId: result.awsAccountId } }, { multi: true })));
        });
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
export function down() {
    return Promise.resolve();
}
