/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
export async function up() {
    const db = this.connection.db;
    const relationshipsStats = db.collection('relationshipstats');
    const resultArr = await relationshipsStats.aggregate([
        {
            $lookup: {
                from: 'relationships',
                localField: 'relationship',
                foreignField: '_id',
                as: 'relationshipData'
            }
        },
        {$unwind: '$relationshipData'},
        {
            $group: {
                _id: '$relationship',
                user: {$first: '$relationshipData.user'},
                status: {$first: '$relationshipData.statusObj'}
            }
        }]).toArray();
    const promises =  resultArr.map(result => {
        const updateQuery =  {
            $set: {
                status: result.status.status,
                user: result.user
            },
            $unset: {statusMessages: ''}
        };
        if (result.status.reason) {
            updateQuery.$set.reason = result.status.reason;
        }
        relationshipsStats.update({relationship: result._id},
            updateQuery, {multi: true});
    });
    await Promise.all(promises);
    const indexes = await relationshipsStats.stats();
    if (indexes.indexDetails.relationship_1_dataBroker_1) {
        await relationshipsStats.dropIndex({ relationship: 1, dataBroker: 1 });
    }
    if (!indexes.indexDetails.relationship_1_dataBroker_1_user_1) {
        await relationshipsStats.createIndex({ relationship: 1, dataBroker: 1, user: 1 });
    }
    const relationships = db.collection('relationships');
    await relationships.update({}, {$unset: {statusObj: ''}}, {multi: true});
}


/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
export function down() {
    return this.model('User').countDocuments();
}
