const {first, compact} = require('lodash');
const {getLogger} = require('../src/utils/logger');
const logger = getLogger();
/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
export async function up() {
    const db = this.connection.db;
    const dataBrokers = db.collection('databrokers');
    const groups = db.collection('groups');
    const dataBrokersWithGroups = await dataBrokers.find({'group' : {$exists: true}}, {group:1, _id:1, user:1}).toArray();
    const dataBrokersToUpdate = await Promise.all(dataBrokersWithGroups.map(async dataBroker => {
        try {
            const groupId = await groups.find({name: dataBroker.group, user: dataBroker.user}, {'_id': 1}).toArray();
            dataBroker.groupId = first(groupId)._id;
            return dataBroker;
        } catch (err) {
            logger.info('error when trying to find group of databroker', dataBroker, err);
        }
    }));
    await Promise.all(compact(dataBrokersToUpdate).map(async dataBrokerDetails => dataBrokers.update({'_id': dataBrokerDetails._id}, {$set: {groupId: dataBrokerDetails.groupId}})));
    await groups.update({}, {$rename : {'databrokers': 'dataBrokers'}}, {multi: true});
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
export async function down() {
    return this.model('User').countDocuments();
}
