require('@babel/register')({
    retainLines: true,
    sourceMap: 'inline',
    presets: ['@babel/preset-env'],
    plugins : ['@babel/plugin-transform-runtime']
});

global.__WEBPACK__ = false;

const Promise = require('bluebird');
const mongoose = require('mongoose');
const config = require('config');
const commandLineArgs = require('command-line-args');
const migration = require('../src/utils/migration').default;

const options = commandLineArgs([
    {name: 'run', alias: 'r', type: Boolean},
    {name: 'create', alias: 'c', type: String},
    {name: 'list', alias: 'l', type: Boolean},
    {name: 'direction', alias: 'd', type: String, defaultValue: 'up'},
    {name: 'name', alias: 'n', type: String}
]);

mongoose.Promise = Promise; // use bluebird with mongoose
mongoose.connect(config.get('db'), async err => {
    if (err) {
        return console.error('Failed connecting to mongo:', err.message);
    }
    let commandPromise = undefined;

    if (options.run !== undefined) {
        commandPromise = migration.run(options.direction, options.name);
    } else if (options.create !== undefined) {
        commandPromise = migration.create(options.name);
    } else {
        commandPromise = migration.list();
    }

    try {
        await commandPromise;
        console.info('Done :-)');
    } catch (err) {
        console.error('Failed with error:', err);
    } finally {
        process.exit();
    }
});


