require('@babel/register')({
    retainLines: true,
    sourceMap: 'inline',
    presets: ['@babel/preset-env'],
    plugins : ['@babel/plugin-transform-runtime']
});

require('../../version');

global.__WEBPACK__ = false;

const server = require('./app').default;
const DataBroker = require('./models/data-broker').default;
const Relationship = require('./models/relationship').default;
const User = require('./models/user').default;

//preload all AWS services, since mock-aws doesn't know to work with lazy loaded service
for (const service in global.AWS) {
    const instance = global.AWS[service];
    delete global.AWS[service];
    global.AWS[service] = instance;
}

// put mock AWS SDK instead of real one
global.AWS = require('mock-aws');

// load mocks
const sqsMock = require('./simulator/sqs-mock');
require('./simulator/ipa-mock');
require('./simulator/marketplace-metering-mock');
require('./simulator/metadata-mock');
require('./simulator/s3-mock');
require('./simulator/sns-mock');
require('./simulator/iam-mock');
require('./simulator/lambda-mock');

function startDataBroker(dataBroker) {
    const { name } = dataBroker;
    if (!['Waiting', 'asterixWaiting'].includes(name)) {
        sqsMock.sendPlacementMessage(dataBroker);
    }
}

// start all data brokers
setImmediate(() => DataBroker.find().exec().map(dataBroker => startDataBroker(dataBroker)));

setInterval(async() => {
    // send data broker heart beat
    const dataBrokers = await DataBroker.find({statusesQueueUrl: {$exists: true}}).exec();
    const promises = dataBrokers.map(async dataBroker => {
        const {name: dataBrokerName} = dataBroker;
        if (!dataBroker.hostname) {
            startDataBroker(dataBroker);
        } else {
            if (['FailedDataBrokerAfterRelationship', 'asterixFailedDataBrokerAfterRelationship'].includes(dataBrokerName)) {
                const relationship = await Relationship.findOne({dataBroker}).exec();
                // do not send heart beat to this data broker once a relationship exists
                if (relationship) {
                    return;
                }
            }
            if (['FAILED', 'Unknown'].includes(dataBrokerName)) {
                return;
            }
            sqsMock.sendHealthMessage(dataBroker);
        }
    });
    await Promise.all(promises);
}, 5000);

// unsubscribe specific user after 1 minute
setInterval(async() => {
    const users = await User.find({
        email: 'unsubscribed@netapp.com',
        'subscription.status': 'subscribe-success'
    }).exec();
    users.map(user => sqsMock.sendSaaSSubscriptionMessage(user.subscription.customerIdentifier, 'unsubscribe-success'));
}, 60000);

module.exports = server;
