import Promise from 'bluebird';
import mongoose from 'mongoose';
import {DB_CONNECTION_STRING} from './utils/consts';
import {getLogger} from './utils/logger';
import migration from './utils/migration';

const logger = getLogger();

mongoose.Promise = Promise; // use bluebird with mongoose

async function connectToMongo() {
    try {
        logger.info('Connecting to DB:', DB_CONNECTION_STRING);
        await mongoose.connect(DB_CONNECTION_STRING,
            {
                reconnectTries: 10,
                reconnectInterval: 1000,
                poolSize: 10000,
                autoIndex: true,
                useNewUrlParser: true,
                useFindAndModify: false,
                useCreateIndex: true
            }
        );
        logger.info('Connected to db');
        mongoose.connection.on('error', err => logger.error('Database connection failure:', err));
    } catch (err) {
        logger.error('Failed connecting to mongo', err);
        process.exit(-1);
    }
}

export async function connectAndMigrate() {
    await connectToMongo();
    try {
        logger.debug('Migrating DB');
        await migration.run('up', null);
    } catch (err) {
        logger.error('Failed migrating MongoDB:', err);
        process.exit(-1);
    }

}
