import format from 'string-template';
import path from 'path';
import {SERVICE_REGION, NEW_DATA_BROKER_POLICY_DOCUMENT} from '../utils/consts';
import {timeout} from '../utils/utils';

const fsPromises = require('fs').promises;

let iam;

function getIam() {
    if (!iam) {
        iam = new AWS.IAM({region: SERVICE_REGION});
    }
    return iam;
}

export async function putUserPolicyAsync(PolicyDocument, PolicyName, UserName) {
    return getIam().putUserPolicy({PolicyDocument, PolicyName, UserName}).promise();
}

export async function createPolicyDocument(commandsQueueArn, statusesQueueArn) {
    const document = await fsPromises.readFile(path.resolve(__dirname, NEW_DATA_BROKER_POLICY_DOCUMENT), 'utf-8');
    return format(document, {
        commandsQueueArn,
        statusesQueueArn
    });
}

export async function createUserAsync(userName) {
    const data = await getIam().createUser({UserName: userName}).promise();
    return data.user;
}

export async function getUserArn(userName) {
    const data = await getIam().getUser({UserName: userName}).promise();
    return data.User.Arn;
}

export async function createAccessKey(userName) {
    const data = await getIam().createAccessKey({UserName: userName}).promise();
    return data.AccessKey;
}

async function listAccessKeys(userName) {
    return getIam().listAccessKeys({UserName: userName}).promise();
}

async function deleteAccessKey(userName, accessKeyId) {
    return getIam().deleteAccessKey({AccessKeyId: accessKeyId, UserName: userName}).promise();
}

export async function deleteKeys(userName) {
    const data = await listAccessKeys(userName);
    return Promise.all(data.AccessKeyMetadata.map(accessKey => deleteAccessKey(userName, accessKey.AccessKeyId)));
}

export async function deleteUserPolicy(userName, policyName) {
    return getIam().deleteUserPolicy({PolicyName: policyName, UserName: userName}).promise();
}

export async function deleteUser(userName) {
    return getIam().deleteUser({ UserName: userName}).promise();
}

//TODO un-comment when new AWS-mock will be ready
/*export function waitForUserExists(userName) {
    return getIam().waitForAsync('userExists', {UserName: userName});
}*/
export async function waitForUserExists(userName, counter = 0) {
    try {
        return getUserArn(userName);
    } catch (e) {
        await timeout(1000);
        if (counter < 20) {
            return waitForUserExists(userName, counter + 1);
        } else {
            throw e;
        }
    }
}