import axios from 'axios';
import { IPA_SERVER_ADDRESS } from '../utils/consts';
import { BadRequest } from '../utils/errors';

export function getLicenseAsync(nssUser, nssPassword, serialNumber) {
    const data = {licenseRequest: {version: '1', serialNumber, op: 'get', package: 'CLOUD-SYNC-AWS', product: 'CLOUDSYNC' }};
    const options = {
        auth: {
            username: nssUser,
            password: nssPassword
        },
        //validateStatus: status => status >= 200 && status <= 500
    };
    return axios.post(`${IPA_SERVER_ADDRESS}/prodstatus/v4`, data, options)
        .then(response => response.data)
        .catch(() => {
            throw new BadRequest('IPA Server is unavailable');
        });

}

export function authenticateAsync(nssUser, nssPassword) {
    const options = {
        auth: {
            username: nssUser,
            password: nssPassword
        },
        validateStatus: status => status >= 200 && status <= 500
        // the error we got from IPA is 500 when having invalid NSS credentials.
        // In any other status code than 200<=statusCode<=500 we get error
    };

    return axios.post(`${IPA_SERVER_ADDRESS}/authenticate/v4`, '', options)
        .then(response => response.status)
        .catch(() => {
            throw new BadRequest('IPA Server is unavailable');
        });
}


