import { PORTAL_LAMBDA__ARN_PRODUCTION, PORTAL_LAMBDA__ARN_TESTING, SERVICE_REGION } from '../utils/consts';
import { isProduction } from '../../../environment';
import {getLogger} from '../utils/logger';
const logger = getLogger();

let lambda;

function getLambda() {
    if (!lambda) {
        lambda = new AWS.Lambda({region: SERVICE_REGION});
    }
    return lambda;
}

async function invokeLambdaFunction(functionArn, payload) {
    const lambdaClient = getLambda();
    return lambdaClient.invoke({
        FunctionName: functionArn,
        Payload: payload
    }).promise();
}

export default async function updatePortalLambda(portalId) {
    try {
        const functionArn =  isProduction() ? PORTAL_LAMBDA__ARN_PRODUCTION : PORTAL_LAMBDA__ARN_TESTING;
        const response = await invokeLambdaFunction(functionArn, JSON.stringify({ type: 'CREATE-RELATIONSHIP', userId : portalId}));
        logger.info('lambda response', response);
    } catch (e) {
        if (e.code === 'AccessDeniedException') {
            logger.warn(e.message);
        } else {
            throw e;
        }
    }
}
