import {LOCAL_STORAGE_REQUEST_ID_KEY, LOCAL_STORAGE_NAMESPACE, LOCAL_STORAGE_REQUEST_KEY,
    LOCAL_STORAGE_ACTION_KEY, LOCAL_STORAGE_RESPONSE_KEY} from '../utils/consts';
import { requestId } from '../utils/utils';

const ns = LOCAL_STORAGE_NAMESPACE;

export default function(req, res, next) {
    const rid = requestId();
    req.id = rid;
    res.set('X-DFIO-Req-Id', rid);

    ns.bindEmitter(req);
    ns.bindEmitter(res);

    ns.run(() => {
        ns.set(LOCAL_STORAGE_REQUEST_ID_KEY, rid);
        ns.set(LOCAL_STORAGE_REQUEST_KEY, req);
        ns.set(LOCAL_STORAGE_ACTION_KEY, `${req.method} ${req.originalUrl}`);
        ns.set(LOCAL_STORAGE_RESPONSE_KEY, res);
        next();
    });
}

export function inContext(action, fn) {
    const rid = requestId();
    const context = {
        [LOCAL_STORAGE_REQUEST_ID_KEY]: rid,
        [LOCAL_STORAGE_ACTION_KEY]: action
    };
    const wrapped = ns.bind(fn, context);
    ns.run(() => {
        wrapped(rid);
    });
}
