import { ExtendedError } from '../utils/errors';
import {getLogger} from '../utils/logger';
const logger = getLogger();

export default function(err, req, res, next) {
    if (err.failedValidation === true) {
        const logMessageParts = [ err.message, '\n' ];
        const addMessage = (type, details) => {
            logMessageParts.push(`${type}: ${details.message}`);
            if (details.path.length > 0) {
                logMessageParts.push(`: ${details.path.join('')}\n`);
            } else {
                logMessageParts.push('\n');
            }
        };
        if (err.results) {
            if (err.results.errors) {
                err.results.errors.forEach(e => {
                    addMessage('ERROR', e);
                });
            }
            if (err.results.warnings) {
                err.results.warnings.forEach(w => {
                    addMessage('WARNING', w);
                });
            }
        }
        err.message = logMessageParts.join('');
        if (req.hasOwnProperty('originalResponse')) {
            err.statusCode = 500;
            try {
                logMessageParts.push(`ORIGINAL RESPONSE BODY:\n${JSON.stringify(JSON.parse(req.originalResponse.toString()), null, 4)}`);
            } catch (e) {/* DO NOTHING */}
        } else {
            err.statusCode = 400;
            try {
                logMessageParts.push(`ORIGINAL REQUEST BODY:\n${JSON.stringify(req.body, null, 4)}`);
            } catch (e) {/* DO NOTHING */}
        }
        logger.error('EXCEPTION HANDLER:', logMessageParts.join(''));
    } else {
        logger.log((err instanceof ExtendedError) ? 'debug' : 'error', 'EXCEPTION HANDLER:', err);
    }
    const status = err.statusCode || err.status || 500;
    if (req.timeline) {
        req.timeline.failureMessage = err.message;
    }
    if (req.timelineV2) {
        req.timelineV2.failureMessage = err.message;
    }
    res.status(status).json({ code: err.errorCode || status, message: err.message });
    next();
}