import onFinished from 'on-finished';
import { isEmpty } from 'lodash';
import Timeline from '../models/timeline';
import TimelineV2 from '../models/timeline-v2';

import {getLogger} from '../utils/logger';
const logger = getLogger();

export default async function(req, res, next) {
    onFinished(res, async(err, res) => {
        const {req: {timeline, timelineV2}} = res;
        if (timeline && timelineV2) {
            if (!timeline.isModified()) {
                logger.warn(`Make sure you haven't forgot to update the timeline for the action ${timeline.summary}`);
            }
            if (!timelineV2.isModified()) {
                logger.warn(`Make sure you haven't forgot to update the timelineV2 for the action ${timeline.summary}`);
            }
            if (timeline.failureMessage || timelineV2.failureMessage) {
                timeline.status = 'FAILED';
                timelineV2.status = 'FAILED';

            } else if (res.statusCode !== 202) { //when status is 202 it is an async request. wait for the message to update the status to done
                timeline.status = 'DONE';
                timelineV2.status = 'DONE';
            }

            timelineV2.markModified('status');
            await Promise.all([timelineV2.save(), timeline.save()]);

        }
    });
    const {method, user} = req;
    try {
        const userExists = !isEmpty(user);
        if (method !== 'GET' && userExists) {

            const {swagger: {operation: {summary}}, user, id: requestId} = req;
            if (summary) {
                req.timeline = new Timeline({user, requestId, summary});
                await req.timeline.save();
                req.timelineV2 = new TimelineV2({user, requestId, summary});
                await req.timelineV2.save();
            }
        }
    } catch (err) {
        // logger.error(`Unable to create a timeline for user ${user} requestId ${requestId} summary ${summary}`);
        next();
    } finally {
        next();
    }
}
