import mongoose from 'mongoose';

const ByolSchema = new mongoose.Schema({
    reservedRelationships: { type: Number, required: true, default: 0},
    usedRelationships: { type: Number, required: true, default: 0}
}, { _id : false });

const costSchema = new mongoose.Schema({
    user:           { type: mongoose.Schema.Types.ObjectId, ref: 'User', required: true, index: true },
    tier1:          {type:  Number, required: true, default: 0},
    tier2:          {type:  Number, required: true, default: 0},
    tier3:          {type:  Number, required: true, default :0},
    byol:           { type: ByolSchema, required: true, default : {enabledRelationship: 0, usedRelationship: 0} },
    timestamp: {type: Date, required: true}
}, {
    timestamps: true
});

//Returns the latest cost records from the table
costSchema.statics.lastCostRecords = function() {
    return this.aggregate([
        {$sort: {timestamp : -1}},
        {$group: { _id: '$user', tier1: {$first: '$tier1'}, tier2 :{$first: '$tier2'}, tier3: {$first: '$tier3'}, byol: {$first: '$byol'}, lastDate: {$max: '$timestamp'}}}]);
};

costSchema.methods.getAllTimeStampPerUser = function(userId) {
    return this.aggregate(
        [ { $match : { user : userId } },
            { $group: {_id: '$user', Dates : { $push: '$timestamp' }}}]
    );
};
export default mongoose.model('Cost', costSchema);
