import mongoose from 'mongoose';

const mongooseAutoIncrement = require('mongoose-sequence')(mongoose);

import {getLogger} from '../utils/logger';
const logger = getLogger();
logger.addContext('module', 'message-model');

const ModelName = 'Message';

const messageSchema = new mongoose.Schema({
    queue:          { type: String, required: true, index: true },
    payload:        { type: mongoose.Schema.Types.Mixed, required: true }
}, {
    _id: false,
    timestamps: true,
    capped: { size: 10485760 } // 10MB
});

// capped collection parameters are not updated (if changed) after collection creation
// run the following command from mongodb shell in order to update the configuration
// db.runCommand({"convertToCapped": "messages", size: 10485760});

messageSchema.plugin(mongooseAutoIncrement);

messageSchema.statics.SendAsync = function(agentId, clientId, requestId, type, payload) {
    return this.QueueAsync(clientId, {
        header: {
            agentId,
            clientId,
            requestId
        },
        payload: {
            [type]: payload
        }
    });
};

messageSchema.statics.QueueAsync = function(queue, payload) {
    logger.debug('Queuing new message. Queue:', queue, '. Payload:', payload);
    return this.create({
        queue,
        payload
    });
};

const Message = mongoose.model(ModelName, messageSchema);

// insert initial document or otherwise tailable cursor is not possible
Message.findOne().exec().then(doc => {
    if (!doc) {
        new Message({queue: 'Capped Collection Initial Document', payload: {}, handled: true }).save();
    }
});

export default Message;
