import mongoose from 'mongoose';
import { defaultTransform } from '../utils/utils';
import Message from './message';

const endPointSchema = new mongoose.Schema({
    protocol:   {type: String, enum: ['nfs', 'cifs', 's3', 'sgws', 'local', 'azure', 'gcp', 'snfs'], required: true},
    nfs:    {type: mongoose.Schema.Types.Mixed, required: false},
    cifs:   {type: mongoose.Schema.Types.Mixed, required: false},
    s3:     {type: mongoose.Schema.Types.Mixed, required: false},
    sgws:   {type: mongoose.Schema.Types.Mixed, required: false},
    local:  {type: mongoose.Schema.Types.Mixed, required: false},
    azure:  {type: mongoose.Schema.Types.Mixed, required: false},
    gcp:    {type: mongoose.Schema.Types.Mixed, required: false},
    snfs:   {type: mongoose.Schema.Types.Mixed, required: false}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => {
            return ret;
        })
    }
});

const relationshipSchema = new mongoose.Schema({
    source:         {type: endPointSchema, required: true},
    target:         {type: endPointSchema, required: true},
    relationshipId: {type: mongoose.Schema.Types.ObjectId, required: true},
    activity:       {type: mongoose.Schema.Types.Mixed, requires: false},
    group:          {type: mongoose.Schema.Types.ObjectId, required: false},
    dataBrokerId:   {type: mongoose.Schema.Types.ObjectId, required: false},
    isQstack:       {type: Boolean, required: true},
    isCvo:          {type: Boolean, required: true, default: false},
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => {
            return ret;
        })
    }
});

const dataBrokerSchema = new mongoose.Schema({
    dataBrokerId: { type: mongoose.Schema.Types.ObjectId, required: true},
    name: { type: String, required : false },
    groupId: {type: mongoose.Schema.Types.ObjectId,  required: false},
    type: {type:String, require:false}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => {
            return ret;
        })
    }
});

const groupSchema = new mongoose.Schema({
    groupId: { type: mongoose.Schema.Types.ObjectId, required: true},
    name: { type: String, required : true }
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => {
            return ret;
        })
    }
});


const timelinev2Schema = new mongoose.Schema({
    user:           { type: mongoose.Schema.Types.ObjectId, ref: 'User', required: true, index: true}, //Do we need to change it to string since we added the option to delete user?
    requestId:      { type: String, required: true },
    status:         { type: String, required: true, default: 'PROGRESS' },
    summary:        { type: String, required: true },
    relationship:   { type: relationshipSchema, required: false },
    dataBroker:     { type: dataBrokerSchema, required: false },
    group:          { type: groupSchema, required: false},
    failureMessage: { type: String, required: false},
}, {
    timestamps: true,
    toJSON: {
        transform: (doc, ret) => {
            ret.createdAt = ret.createdAt.getTime();
            delete ret.updatedAt;
            delete ret['__v'];
            delete ret.user;
            delete ret['_id'];
            ret.id = doc.id;
            return ret;
        }
    }
});

timelinev2Schema.index({ 'relationship.relationshipId': 1, status: 1, summary: 1 }, {sparse: true});
timelinev2Schema.index({ 'dataBroker.dataBrokerId': 1, status: 1, summary: 1  }, {sparse: true}); //TODO remove index - there are no searches by databroker

timelinev2Schema.post('save', (doc) => {
    Message.SendAsync(undefined, doc.user.id, doc.requestId, 'timeline-v2', doc.toJSON());
});

export default mongoose.model('TimelineV2', timelinev2Schema);
