import mongoose from 'mongoose';
import Message from './message';

const timelineSchema = new mongoose.Schema({
    user:           { type: mongoose.Schema.Types.ObjectId, ref: 'User', required: true, index: true },
    requestId:      { type: String, required: true, index: true },
    status:         { type: String, required: true, index: true, default: 'PROGRESS'},
    summary:        { type: String, required: true },
    source:         { type: String, required: false},
    target:         { type: String, required: false},
    dataBroker:     { type: String, required: false},
    failureMessage: { type: String, required: false},
    activity:       {type: mongoose.Schema.Types.Mixed, requires: false}
}, {
    timestamps: true,
    toJSON: {
        transform: (doc, ret) => {
            ret.createdAt = ret.createdAt.getTime();
            delete ret.updatedAt;
            delete ret['__v'];
            delete ret.user;
            delete ret['_id'];
            ret.id = doc.id;
            return ret;
        }
    }
});

timelineSchema.post('save', (doc) => {
    Message.SendAsync(undefined, doc.user.id, doc.requestId, 'timeline', doc.toJSON());
});

export default mongoose.model('Timeline', timelineSchema);
