import express from 'express';
import { concat, orderBy, uniqBy} from 'lodash';
import Relationship from '../models/relationship';

const router = express.Router();

const BASE_URL = '/storages';

function findProtocol(type) {
    switch (type) {
        case 'ibm' :
            return 'sgws';
        case 'anf':
        case 'cvs':
        case 'efs':
            return 'nfs';
        default:
            return type;
    }
}


async function queryRelationshipByType(endPoint, key, type, req) {

    const defineHost = (relationship, hostType) => {
        return relationship[endPoint][hostType][key];
    };

    const dbType = findProtocol(type);
    const {user} = req;
    let query;

    if (['ibm', 'sgws'].includes(type)) {
        query = {user, [`${endPoint}.${dbType}.${key}`]: {$exists: true}, [`${endPoint}.${dbType}.objectStorageType`] : type};
    } else if (['cvs', 'anf', 'efs'].includes(type)) {
        query = {user, [`${endPoint}.${dbType}.${key}`]: {$exists: true}, [`${endPoint}.${dbType}.nfsType`]: type};
    } else if (type === 'nfs') {
        query = {user, [`${endPoint}.${dbType}.${key}`]: {$exists: true}, [`${endPoint}.${dbType}.nfsType`] : {$in: ['nfs', 'snfs']}};
    } else {
        query = {user, [`${endPoint}.${dbType}.${key}`] : {$exists: true}};
    }
    const relationships = await Relationship.find(query).exec();
    return relationships.map(relationship => {
        const hostName = defineHost(relationship, dbType);
        return {
            id: relationship.id,
            host: hostName
        };
    });
}

async function sendUniqueByKey(type, key, req, res, next) {

    try {
        const [sourceHosts, targetHosts] = await Promise.all([queryRelationshipByType('source', key, type, req), queryRelationshipByType('target', key, type, req)]);
        const relationships = concat(sourceHosts, targetHosts);
        const uniqueRelationships = uniqBy(relationships, 'host');
        const orderedRelationships = orderBy(uniqueRelationships, [key], ['asc']);
        res.json(orderedRelationships);
    } catch (e) {
        next(e);
    }
}

router.get(`${BASE_URL}/:protocol`, (req, res, next) => {
    const {params: {protocol} } = req;
    if (protocol === 'gcp') {
        return sendUniqueByKey(protocol, 'projectId', req, res, next);
    } else if (protocol === 'azure') {
        return sendUniqueByKey(protocol, 'storageAccountName', req, res, next);
    } else {
        return sendUniqueByKey(protocol, 'host', req, res, next);
    }
});

export default router;