import EventEmitter from 'events';
import { DATA_BROKER_BUNDLE_LABEL } from '../../src/utils/consts';

function findBucketLocation(bucket) {
    let region;
    switch (bucket) {
        case 'small' :
            region = 'us-west-2';
            break;
        case 'medium':
            region = 'us-west-1';
            break;
        case 'tiny':
            region = 'us-east-1';
            break;
        case 'joey':
            region = 'ap-northeast-1';
            break;
        case 'phoebe':
        case 'large':
        case 'rachel':
        case 'fabric-pool-a1':
            region = 'eu-central-1';
            break;
        case 'chandler':
        case 'monica':
            region = 'ap-south-1';
            break;
        case 'ross':
            region = 'ca-central-1';
            break;
        default:
            region = 'us-east-1';
    }
    return region;
}

AWS.mock('S3', 'putObject', () => ({promise: () => ({})}));

AWS.mock('S3', 'headBucket', params => {
    class MyEmitter extends EventEmitter {}
    const myEmitter = new MyEmitter();
    myEmitter.send = () => {};
    const region = findBucketLocation(params.Bucket);
    setTimeout(() => myEmitter.emit('httpHeaders', 200, {'x-amz-bucket-region': region}), 3000);
    return myEmitter;
});

AWS.mock('S3', 'listObjectsV2', params => ({
    promise: () => {
        const {Bucket, Prefix} = params;
        const localVersion = /^(.+)\s.*/.exec(global.__VERSION__)[1];
        switch (Bucket) {
            case 'small':
                if (!Prefix) {
                    return {CommonPrefixes: [{Prefix: 'small1/'}]};
                } else if (Prefix === 'small1/') {
                    return {CommonPrefixes: [{Prefix: 'small2/'}, {Prefix: 'small3/'}]};
                } else if (Prefix === 'small1/small2/') {
                    return {CommonPrefixes: []};
                } else if (Prefix === 'small1/small3/') {
                    return {CommonPrefixes: []};
                }
                break;
            case 'medium':
                if (!Prefix) {
                    return {CommonPrefixes: [{Prefix: 'medium1/'}]};
                } else if (Prefix === 'medium1/') {
                    return {CommonPrefixes: [{Prefix: 'medium2/'}, {Prefix: 'mediumedium3/'}]};
                } else if (Prefix === 'medium1/medium2/') {
                    return {CommonPrefixes: []};
                } else if (Prefix === 'medium1/medium3/') {
                    return {CommonPrefixes: []};
                }
                break;
            case 'large':
                return {CommonPrefixes: []};
            case 'metadata.datafabric.io':
                return {
                    Contents: [{
                        Key: `data-mover-update-bundle/dfio-agent-update-bundle_${localVersion}.zip`,
                        LastModified: '2016-07-12T07:32:58.000Z', Size: 12209903
                    },
                    {
                        Key: `data-mover-update-bundle/phoenix/dfio-agent-update-bundle_${localVersion}.zip`,
                        LastModified: '2016-07-12T07:32:58.000Z', Size: 12209903
                    }], CommonPrefixes: []
                };
            default:
                throw new Error(`Bucket ${params.Bucket} not found`);
        }
    }
}));

AWS.mock('S3', 'getObjectTagging', () => ({
    promise: () => {
        const TagSet = [
            {
                Key: 'environment',
                Value: 'production'
            },
            {
                Key: 'version',
                Value: global.__VERSION__
            }
        ];
        const labelValue = DATA_BROKER_BUNDLE_LABEL;
        if (labelValue) {
            TagSet.push({
                Key: 'label',
                Value: labelValue
            });
        }
        return {TagSet};
    }}));
