import Agenda from 'agenda';
import mongoose from 'mongoose';
import os from 'os';
import {AGENDA_PROCESS_EVERY, AGENDA_COLLECTION_NAME, AGENDA_TASK_INTERVAL_UPDATE_DATABROKER,
    AGENDA_TASK_NAME_UPDATE_DATABROKER, AGENDA_TASK_NAME_SYNC_SCHEDULER, AGENDA_TASK_INTERVAL_BACKUP_DB, AGENDA_TASK_NAME_BACKUP_DB,
    AGENDA_TASK_INTERVAL_SAAS_METERING, AGENDA_TASK_NAME_SAAS_METERING, AGENDA_TASK_INTERVAL_SYNC_SCHEDULER} from '../utils/consts';
import {getLogger} from '../utils/logger';
const logger = getLogger();
const agenda = new Agenda({
    name: `${os.hostname()}-${process.pid}`,
    processEvery: AGENDA_PROCESS_EVERY
});

// hack Agenda not to propagate exception and kill our server in case of DB connection error
/*eslint no-underscore-dangle: ["error", { "allow": ["_findAndLockNextJob", "_findAndLockNextJobOrig", "_processInterval"] }]*/
agenda._findAndLockNextJobOrig = agenda._findAndLockNextJob;
agenda._findAndLockNextJob = function(jobName, definition, cb) {
    try {
        this._findAndLockNextJobOrig(jobName, definition, cb);
    } catch (e) {
        logger.warn('Agenda error:', e.message);
    }
};

// hack Agenda not to crash when calling stop on non started instance
agenda.stopOrig = agenda.stop;
agenda.stop = function(cb) {
    if (this._processInterval) {
        this.stopOrig(cb);
    } else {
        cb();
    }
};

agenda.on('ready', () => {
    agenda.every(AGENDA_TASK_INTERVAL_BACKUP_DB, AGENDA_TASK_NAME_BACKUP_DB);
    agenda.every(AGENDA_TASK_INTERVAL_SAAS_METERING, AGENDA_TASK_NAME_SAAS_METERING);
    agenda.every(AGENDA_TASK_INTERVAL_SYNC_SCHEDULER, AGENDA_TASK_NAME_SYNC_SCHEDULER);
    agenda.every(AGENDA_TASK_INTERVAL_UPDATE_DATABROKER, AGENDA_TASK_NAME_UPDATE_DATABROKER);

    agenda.start();
});

agenda.on('error', err => {
    logger.error('Got agenda error', err);
});

export async function initAgenda() {

    await new Promise((resolve, reject) => {
        mongoose.connection.db.collection(AGENDA_COLLECTION_NAME, (err, agendaCollection) => {
            if (err) {
                return reject(err);
            }
            resolve(agendaCollection.deleteMany({}));
        });
    });

    await agenda.mongo(mongoose.connection.db, AGENDA_COLLECTION_NAME, err => {
        if (err) {
            throw err;
        }
    });
}

export default agenda;
