import conf from 'config';
import fs from 'fs';
import path from 'path';
import log4js from 'log4js';
import moment from 'moment';
import {first} from 'lodash';
import stackTrace from 'stack-trace';
import * as consts from './consts';

const logsDirectory = 'logs';
if (!fs.existsSync(logsDirectory)) {
    fs.mkdirSync(logsDirectory);
}

const config = JSON.parse(fs.readFileSync(path.resolve(__dirname, consts.LOG4JS_FILE), 'utf8'));

const tokens = {
    date: (event) => moment(event.startTime).format('Y-MM-DDTHH:mm:ss.SSSZZ'),
    requestId: () => consts.LOCAL_STORAGE_NAMESPACE.get(consts.LOCAL_STORAGE_REQUEST_ID_KEY) || 'headless',
    who: () => {
        const req = consts.LOCAL_STORAGE_NAMESPACE.get(consts.LOCAL_STORAGE_REQUEST_KEY);
        if (req) {
            const { user, dataBroker } = req;
            if (dataBroker) {
                return dataBroker;
            } else if (user) {
                return user;
            } else {
                return 'anonymous';
            }
        } else {
            return 'system';
        }
    },
    action: () => consts.LOCAL_STORAGE_NAMESPACE.get(consts.LOCAL_STORAGE_ACTION_KEY) || 'system'
};

Object.keys(config.appenders).forEach(appenderType => {
    const {layout} = config.appenders[appenderType];
    if (layout) {
        const {pattern} = layout;
        if (pattern) {
            layout.tokens = tokens;
        }
    }
});


log4js.configure(config);

export function getLogger(category) {
    const logger = log4js.getLogger(category);
    const trace = stackTrace.parse(new Error('something went wrong'));
    logger.addContext('module', trace[1].getFileName().replace(/^.*[\\\/]src[\\\/]/, '').split(/[\\\/]/).join('.'));
    logger.addContext('service', first(conf.get('server.host').split('.')));
    return logger;
}

export function shutdown(cb) {
    log4js.shutdown(cb);
}