import swaggerTools from 'swagger-tools';

import {getLogger} from '../utils/logger';
const logger = getLogger();

export function validateSwaggerSpec(swaggerSpec) {
    return new Promise((resolve, reject) => {
        swaggerTools.specs.v2_0.validate(swaggerSpec, (err, results) => {
            if (err) {
                logger.error('Swagger Spec validation failed:', err);
                reject(err);
            }

            results = results || { errors: [], warnings: [] };

            function formatAndJoin(resultArray) {
                return resultArray.map(r => `${r.path}:${r.message}`).join('\n\t');
            }

            if (results.errors.length > 0) {
                logger.error('>>>>>>>>>>>>>', JSON.stringify(results));
                logger.error(`Swagger Spec Errors:\n\t${formatAndJoin(results.errors)}`);
                reject(new Error('Swagger Spec validation errors detected'));
            }

            if (results.warnings.length > 0) {
                logger.warn(`Swagger Spec Warnings:\n\t${formatAndJoin(results.warnings)}`);
            }

            logger.debug(`Swagger Spec validation completed: ${results.errors.length} errors, ${results.warnings.length} warnings`);
            resolve(swaggerSpec);
        });
    });
}