const webpack = require('webpack');
const path = require('path');
const nodeExternals = require('webpack-node-externals');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const version = require('../version');

module.exports = {
    target: 'node',
    optimization: {
        nodeEnv: false
    },
    mode: 'none',
    node: {
        __dirname: false
    },
    cache: true,
    devtool: 'source-map',
    entry: {
        app: './src/app.js'
    },
    module: {
        rules: [
            {
                enforce: 'pre',
                test: /\.js?$/,
                exclude: /node_modules/,
                loader: 'eslint-loader',
                options: {
                    quiet: true,
                    failOnError: true,
                    failOnWarning: false,
                    emitError: false,
                    emitWarning: false
                }
            },
            {
                test: /\.js$/,
                exclude: /node_modules/,
                use: {
                    loader: 'babel-loader',
                    options: {
                        presets: ['@babel/preset-env']
                    }
                }
            },
            {
                test: /\.json$/,
                loader: 'json'
            }
        ]
    },
    output: {
        path: path.join(__dirname, 'build'),
        filename: '[name].js'
    },
    externals: [nodeExternals()],
    plugins: [
        new CleanWebpackPlugin(),
        new webpack.DefinePlugin({
            __VERSION__: JSON.stringify(version()),
            __WEBPACK__: JSON.stringify(true)
        }),
        new webpack.BannerPlugin({banner: 'require("source-map-support").install();', raw: true, entryOnly: false}),
        new CopyWebpackPlugin([
            {from: 'package.json', to: 'package.json'},
            {from: 'src/swagger', to: 'swagger'},
            {from: 'src/aws/newDataBroker-cfn.json', to: 'newDataBroker-cfn.json'},
            {from: 'src/aws/installer-newDataBroker.sh', to: 'installer-newDataBroker.sh'},
            {from: 'src/azure/new-azure-template.json', to: 'new-azure-template.json'},
            {from: 'src/gcp/new-gcp-template.jinja', to: 'new-gcp-template.jinja'},
            {from: 'src/swagger-ui', to: 'swagger-ui'},
            {from: 'src/config', to: 'config', ignore: ['local.json']},
            {from: '../ui/build', to: 'public'},
            {from: 'src/newDataBroker-document-policy.json', to: 'newDataBroker-document-policy.json'},
            {from: 'src/resources', to: 'resources'}
        ])
    ]
};
